
class Arbiter(object):
    """Base class for SNAPstraction Arbiters.

    Gus can use an Arbiter to interface with an external SNAP network traffic manager.
    """

    def acquire(self, priority):
        """Placeholder function for the Arbiter.acquire method.
        This should request access at the given priority from the external SNAP network traffic manager.

        When a packet is ready to be transmitted, Gus will yield on arbiter.acquire(priority).
        The value returned from this call must be yieldable:

         ``` python
           def acquire(self, priority):
               return Future
         ```
           or
         ``` python
           @coroutine
           def acquire(self, priority):
               raise Return(True)
         ```

        If the value yielded is truthy, Gus will proceed with transmission.
        If the value is falsey, Gus will re-enqueue the packet for transmission later.

        Args:
            priority (int): The priority of the request (0x00-0xFF). 0 is the highest priority.

        Raises:
            NotImplementedError: Designed to be a base class. Replace this with your own acquire function.
        """
        raise NotImplementedError

    def release(self):
        """Placeholder function for the Arbiter.release method.
        This tells the external SNAP network traffic manager to release the lock on the queue and return to normal function.

        After acquiring a lock and transmitting a packet, Gus will call (fire-and-forget) arbiter.release().

        Raises:
            NotImplementedError: Designed to be a base class. Replace this with your own acquire function.
        """
        raise NotImplementedError
