from synapse_mock_tools import SynapseFunctionMock
from tornado.gen import coroutine, sleep, Return


class Arbiter(SynapseFunctionMock):

    def __init__(self, *args, **kwargs):

        super(Arbiter, self).__init__((), {}, (), *args, **kwargs)

        self.acquire = _Acquire()
        self.release = _Release()


class _Acquire(SynapseFunctionMock):

    def __init__(self, *args, **kwargs):

        arg_schema = (int,)

        exceptions = Exception

        super(_Acquire, self).__init__(arg_schema, {}, exceptions, *args, **kwargs)

    def returns(self, value=True, delay=None):
        """Arbiter.acquire will return a resolved future with a True/False value in some given time period.

        Args:
            delay (int): Number of seconds to wait before returning future (to emulate a delay in getting a response
                from the master system).
            bool (bool): Bool value you want the acquire call to return packaged as a future.
        """
        @coroutine
        def acquire_result(value, delay):
            if delay:
                yield sleep(delay)
            raise Return(value)

        self.side_effect_list.append(acquire_result(value, delay))
        self.side_effect = self.side_effect_list


class _Release(SynapseFunctionMock):

    def __init__(self, *args, **kwargs):
        super(_Release, self).__init__((), {}, (), *args, **kwargs)

    def returns(self):
        pass
