import mock
import unittest

from snapstraction import build
from snapstraction.util import get_virtualenv_path

import subprocess


class BuildTests(unittest.TestCase):

    @mock.patch('snapbuild.snapbuild.call')
    def test_call_calls_snapbuild(self, cmd):
        build.call('foo.py', 'SN220', '2.6.9')
        cmd.assert_called_with(args=['_', '-v', 'foo.py', '-m', 'SN220', '-c', '2.6.9', '-I', get_virtualenv_path()], stderr=subprocess.STDOUT)

    @mock.patch('snapbuild.snapbuild.call')
    def test_call_passes_includes(self, cmd):
        build.call('foo.py', 'SN220', '2.6.9', includes=['this', 'that', 'the_other'])
        cmd.assert_called_with(args=['_', '-v', 'foo.py', '-m', 'SN220', '-c', '2.6.9', '-I', get_virtualenv_path(), '-I', 'this', '-I', 'that', '-I', 'the_other'], stderr=subprocess.STDOUT)

    @mock.patch('snapbuild.snapbuild.call')
    def test_call_passes_platform(self, cmd):
        build.call('foo.py', 'SN220', '2.6.9', platform='A thing to stand on')
        cmd.assert_called_with(args=['_', '-v', 'foo.py', '-m', 'SN220', '-c', '2.6.9', '-I', get_virtualenv_path(), '-p', 'A thing to stand on'], stderr=subprocess.STDOUT)

    @mock.patch('snapbuild.snapbuild.call')
    def test_call_passes_output_path(self, cmd):
        build.call('foo.py', 'SN220', '2.6.9', output_path='over there')
        cmd.assert_called_with(args=['_', '-v', 'foo.py', '-m', 'SN220', '-c', '2.6.9', '-I', get_virtualenv_path(), '-o', 'over there'], stderr=subprocess.STDOUT)

    @mock.patch('snapbuild.snapbuild.call')
    def test_call_extracts_crc_from_output(self, cmd):
        cmd.return_value = "Blah Blah\nBlah\nSNAPpy CRC: 0xc0de\nblah\nblah blah"
        crc = build.call('foo.py', 'SN220', '2.6.9')
        self.assertEqual(crc, "0xc0de")

    @mock.patch('snapbuild.snapbuild.call')
    def test_call_exits_on_snapbuild_exception(self, cmd):
        cmd.side_effect = subprocess.CalledProcessError(-1, 'snapbuild')
        with self.assertRaises(SystemExit):
            build.call('foo.py', 'SN220', '2.6.9')

    @mock.patch('snapbuild.snapbuild.call')
    def test_call_returns_None_for_crc_on_error(self, cmd):
        cmd.side_effect = Exception()
        crc = build.call('foo.py', 'SN220', '2.6.9')
        self.assertEqual(crc, None)
