"""Tests for SNAPstraction Low-level functions

Assumes: Gus works."""

import mock
import unittest

import snapstraction.low_level

from tornado.testing import AsyncTestCase, gen_test
from tornado.concurrent import Future


def mock_gus():
    gus = mock.MagicMock()
    gus.bridge_address = '000001'

    gus.ucast_rpc.return_value = Future()
    gus.ucast_rpc.return_value.set_result(True)

    gus.mcast_rpc.return_value = Future()
    gus.mcast_rpc.return_value.set_result(True)

    gus.dmcast_rpc.return_value = Future()
    gus.dmcast_rpc.return_value.set_result(True)

    gus.callback_ucast_rpc.return_value = Future()
    gus.callback_ucast_rpc.return_value.set_result(True)

    gus.callback_mcast_rpc.return_value = Future()
    gus.callback_mcast_rpc.return_value.set_result(True)

    gus.callback_dmcast_rpc.return_value = Future()
    gus.callback_dmcast_rpc.return_value.set_result(True)

    gus.ucast_data.return_value = Future()
    gus.ucast_data.return_value.set_result(True)

    gus.mcast_data.return_value = Future()
    gus.mcast_data.return_value.set_result(True)

    gus.callback_ucast_data.return_value = Future()
    gus.callback_ucast_data.return_value.set_result(True)

    gus.callback_mcast_data.return_value = Future()
    gus.callback_mcast_data.return_value.set_result(True)

    return gus


class TestSendFunctions(AsyncTestCase):

    def setUp(self):
        super(TestSendFunctions, self).setUp()

        self.gus = mock_gus()

    def test_ucast_rpc(self):
        snapstraction.low_level.ucast_rpc(self.gus, '123456', 'ping', 'abc', 123)

        self.gus.ucast_rpc.assert_called_once_with('123456', 'ping', 'abc', 123)

    def test_mcast_rpc(self):
        snapstraction.low_level.mcast_rpc(self.gus, 1, 5, 'ping', 'abc', 123)

        self.gus.mcast_rpc.assert_called_once_with(1, 5, 'ping', 'abc', 123)

    def test_dmcast_rpc(self):
        snapstraction.low_level.dmcast_rpc(self.gus, ['123456', '123457'], 1, 5, 50, 'ping', 'abc', 123)

        self.gus.dmcast_rpc.assert_called_once_with(['123456', '123457'], 1, 5, 50, 'ping', 'abc', 123)


class TestCallFunctions(AsyncTestCase):

    def test_callback_ucast_rpc(self):
        gus = mock_gus()

        snapstraction.low_level.callback_ucast_rpc(gus, '123456', 'ping', 'abc', 123)

        gus.callback_ucast_rpc.assert_called_once_with('123456', 'ping', 'abc', 123)

    def test_callback_mcast_rpc(self):
        gus = mock_gus()

        snapstraction.low_level.callback_mcast_rpc(gus, ['123456', '123457'], 'ping', 'abc', 123)

        gus.callback_mcast_rpc.assert_called_once_with(['123456', '123457'], 'ping', 'abc', 123)

    def test_callback_dmcast_rpc(self):
        gus = mock_gus()

        snapstraction.low_level.callback_dmcast_rpc(gus, ['123456', '123457'], 'ping', 'abc', 123, delay_factor=50)

        gus.callback_dmcast_rpc.assert_called_once_with(['123456', '123457'], 'ping', 'abc', 123, delay_factor=50)
